@@ -22,7 +22,7 @@
     #define DECORR_STEREO_PASS_CONT unpack_decorr_stereo_pass_cont_x86
     #define DECORR_STEREO_PASS_CONT_AVAILABLE unpack_cpu_has_feature_x86(CPU_FEATURE_MMX)
     #define DECORR_MONO_PASS_CONT unpack_decorr_mono_pass_cont_x86
-#elif defined(OPT_ASM_X64) && (defined (_WIN64) || defined(__CYGWIN__) || defined(__MINGW64__))
+#elif defined(OPT_ASM_X64) && (defined (_WIN64) || defined(__CYGWIN__) || defined(__MINGW64__) || defined(__midipix__))
     #define DECORR_STEREO_PASS_CONT unpack_decorr_stereo_pass_cont_x64win
     #define DECORR_STEREO_PASS_CONT_AVAILABLE 1
     #define DECORR_MONO_PASS_CONT unpack_decorr_mono_pass_cont_x64win
@@ -43,7 +43,7 @@ extern void DECORR_MONO_PASS_CONT (struct decorr_pass *dpp, int32_t *buffer, int
 
 // This flag provides the functionality of terminating the decoding and muting
 // the output when a lossy sample appears to be corrupt. This is automatic
-// for lossless files because a corrupt sample is unambigious, but for lossy
+// for lossless files because a corrupt sample is unambiguous, but for lossy
 // data it might be possible for this to falsely trigger (although I have never
 // seen it).
 
@@ -52,7 +52,7 @@ extern void DECORR_MONO_PASS_CONT (struct decorr_pass *dpp, int32_t *buffer, int
 ///////////////////////////// executable code ////////////////////////////////
 
 // This monster actually unpacks the WavPack bitstream(s) into the specified
-// buffer as 32-bit integers or floats (depending on orignal data). Lossy
+// buffer as 32-bit integers or floats (depending on original data). Lossy
 // samples will be clipped to their original limits (i.e. 8-bit samples are
 // clipped to -128/+127) but are still returned in longs. It is up to the
 // caller to potentially reformat this for the final output including any
@@ -80,8 +80,14 @@ int32_t unpack_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample_co
     struct decorr_pass *dpp;
     int tcount, m = 0;
 
-    if (wps->sample_index + sample_count > wps->wphdr.block_index + wps->wphdr.block_samples)
-        sample_count = wps->wphdr.block_index + wps->wphdr.block_samples - wps->sample_index;
+    // don't attempt to decode past the end of the block, but watch out for overflow!
+
+    if (wps->sample_index + sample_count > GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples &&
+        (uint32_t) (GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples - wps->sample_index) < sample_count)
+            sample_count = (uint32_t) (GET_BLOCK_INDEX (wps->wphdr) + wps->wphdr.block_samples - wps->sample_index);
+
+    if (GET_BLOCK_INDEX (wps->wphdr) > wps->sample_index || wps->wphdr.block_samples < sample_count)
+        wps->mute_error = TRUE;
 
     if (wps->mute_error) {
         if (wpc->reduced_channels == 1 || wpc->config.num_channels == 1 || (flags & MONO_FLAG))
@@ -94,7 +100,7 @@ int32_t unpack_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample_co
     }
 
     if ((flags & HYBRID_FLAG) && !wps->block2buff)
-        mute_limit *= 2;
+        mute_limit = (mute_limit * 2) + 128;
 
     //////////////// handle lossless or hybrid lossy mono data /////////////////
 
@@ -113,6 +119,9 @@ int32_t unpack_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample_co
         else
             i = get_words_lossless (wps, buffer, sample_count);
 
+        if (i != sample_count)
+            goto get_word_eof;
+
 #ifdef DECORR_MONO_PASS_CONT
         if (sample_count < 16)
             for (tcount = wps->num_terms, dpp = wps->decorr_passes; tcount--; dpp++)
@@ -131,6 +140,9 @@ int32_t unpack_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample_co
             decorr_mono_pass (dpp, buffer, sample_count);
 #endif
 
+#ifndef LOSSY_MUTE
+        if (!(flags & HYBRID_FLAG))
+#endif
         for (bptr = buffer; bptr < eptr; ++bptr) {
             if (labs (bptr [0]) > mute_limit) {
                 i = (uint32_t)(bptr - buffer);
@@ -139,6 +151,11 @@ int32_t unpack_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample_co
 
             crc = crc * 3 + bptr [0];
         }
+#ifndef LOSSY_MUTE
+        else
+            for (bptr = buffer; bptr < eptr; ++bptr)
+                crc = crc * 3 + bptr [0];
+#endif
     }
 
     /////////////// handle lossless or hybrid lossy stereo data ///////////////
@@ -159,6 +176,9 @@ int32_t unpack_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample_co
         else
             i = get_words_lossless (wps, buffer, sample_count);
 
+        if (i != sample_count)
+            goto get_word_eof;
+
 #ifdef DECORR_STEREO_PASS_CONT
         if (sample_count < 16 || !DECORR_STEREO_PASS_CONT_AVAILABLE) {
             for (tcount = wps->num_terms, dpp = wps->decorr_passes; tcount--; dpp++)
@@ -185,12 +205,15 @@ int32_t unpack_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample_co
         if (flags & JOINT_STEREO)
             for (bptr = buffer; bptr < eptr; bptr += 2) {
                 bptr [0] += (bptr [1] -= (bptr [0] >> 1));
-                crc += (crc << 3) + (bptr [0] << 1) + bptr [0] + bptr [1];
+                crc += (crc << 3) + ((uint32_t) bptr [0] << 1) + bptr [0] + bptr [1];
             }
         else
             for (bptr = buffer; bptr < eptr; bptr += 2)
-                crc += (crc << 3) + (bptr [0] << 1) + bptr [0] + bptr [1];
+                crc += (crc << 3) + ((uint32_t) bptr [0] << 1) + bptr [0] + bptr [1];
 
+#ifndef LOSSY_MUTE
+        if (!(flags & HYBRID_FLAG))
+#endif
         for (bptr = buffer; bptr < eptr; bptr += 16)
             if (labs (bptr [0]) > mute_limit || labs (bptr [1]) > mute_limit) {
                 i = (uint32_t)(bptr - buffer) / 2;
@@ -251,10 +274,9 @@ int32_t unpack_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample_co
 
             crc += (crc << 1) + read_word;
 
-#ifdef LOSSY_MUTE
             if (labs (read_word) > mute_limit)
                 break;
-#endif
+
             *bptr++ = read_word;
         }
 
@@ -423,17 +445,17 @@ int32_t unpack_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample_co
                 right = right_c;
             }
 
-#ifdef LOSSY_MUTE
             if (labs (left) > mute_limit || labs (right) > mute_limit)
                 break;
-#endif
-            crc += (crc << 3) + (left << 1) + left + right;
+
+            crc += (crc << 3) + ((uint32_t) left << 1) + left + right;
             *bptr++ = left;
             *bptr++ = right;
         }
     else
         i = 0;  /* this line can't execute, but suppresses compiler warning */
 
+get_word_eof:
     if (i != sample_count) {
         memset (buffer, 0, sample_count * (flags & MONO_FLAG ? 4 : 8));
         wps->mute_error = TRUE;
@@ -676,9 +698,9 @@ static void fixup_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample
 
     if (flags & INT32_DATA) {
         uint32_t count = (flags & MONO_DATA) ? sample_count : sample_count * 2;
-        int sent_bits = wps->int32_sent_bits, zeros = wps->int32_zeros;
-        int ones = wps->int32_ones, dups = wps->int32_dups;
-        uint32_t data, mask = (1 << sent_bits) - 1;
+        int sent_bits = wps->int32_sent_bits & 0x1f, zeros = wps->int32_zeros & 0x1f;
+        int ones = wps->int32_ones & 0x1f, dups = wps->int32_dups & 0x1f;
+        uint32_t data, mask = (1U << sent_bits) - 1;
         int32_t *dptr = buffer;
 
         if (bs_is_open (&wps->wvxbits)) {
@@ -687,15 +709,15 @@ static void fixup_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample
             while (count--) {
 //              if (sent_bits) {
                     getbits (&data, sent_bits, &wps->wvxbits);
-                    *dptr = (*dptr << sent_bits) | (data & mask);
+                    *dptr = ((uint32_t) *dptr << sent_bits) | (data & mask);
 //              }
 
                 if (zeros)
-                    *dptr <<= zeros;
+                    *(uint32_t*)dptr <<= zeros;
                 else if (ones)
-                    *dptr = ((*dptr + 1) << ones) - 1;
+                    *dptr = ((uint32_t)(*dptr + 1) << ones) - 1;
                 else if (dups)
-                    *dptr = ((*dptr + (*dptr & 1)) << dups) - (*dptr & 1);
+                    *dptr = ((uint32_t)(*dptr + (*dptr & 1)) << dups) - (*dptr & 1);
 
                 crc = crc * 9 + (*dptr & 0xffff) * 3 + ((*dptr >> 16) & 0xffff);
                 dptr++;
@@ -719,11 +741,11 @@ static void fixup_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample
 
             while (count--) {
                 if (zeros)
-                    *dptr <<= zeros;
+                    *(uint32_t*)dptr <<= zeros;
                 else if (ones)
-                    *dptr = ((*dptr + 1) << ones) - 1;
+                    *dptr = ((uint32_t)(*dptr + 1) << ones) - 1;
                 else if (dups)
-                    *dptr = ((*dptr + (*dptr & 1)) << dups) - (*dptr & 1);
+                    *dptr = ((uint32_t)(*dptr + (*dptr & 1)) << dups) - (*dptr & 1);
 
                 dptr++;
             }
@@ -732,27 +754,29 @@ static void fixup_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample
             shift += zeros + sent_bits + ones + dups;
     }
 
+    shift &= 0x1f;
+
     if (lossy_flag) {
         int32_t min_value, max_value, min_shifted, max_shifted;
 
         switch (flags & BYTES_STORED) {
             case 0:
-                min_shifted = (min_value = -128 >> shift) << shift;
+                min_shifted = (uint32_t)(min_value = -128 >> shift) << shift;
                 max_shifted = (max_value = 127 >> shift) << shift;
                 break;
 
             case 1:
-                min_shifted = (min_value = -32768 >> shift) << shift;
+                min_shifted = (uint32_t)(min_value = -32768 >> shift) << shift;
                 max_shifted = (max_value = 32767 >> shift) << shift;
                 break;
 
             case 2:
-                min_shifted = (min_value = -8388608 >> shift) << shift;
+                min_shifted = (uint32_t)(min_value = -8388608 >> shift) << shift;
                 max_shifted = (max_value = 8388607 >> shift) << shift;
                 break;
 
             case 3: default:    /* "default" suppresses compiler warning */
-                min_shifted = (min_value = (int32_t) 0x80000000 >> shift) << shift;
+                min_shifted = (uint32_t)(min_value = (int32_t) 0x80000000 >> shift) << shift;
                 max_shifted = (max_value = (int32_t) 0x7fffffff >> shift) << shift;
                 break;
         }
@@ -766,7 +790,7 @@ static void fixup_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample
             else if (*buffer > max_value)
                 *buffer++ = max_shifted;
             else
-                *buffer++ <<= shift;
+                *(uint32_t*)buffer++ <<= shift;
         }
     }
     else if (shift) {
@@ -774,7 +798,7 @@ static void fixup_samples (WavpackContext *wpc, int32_t *buffer, uint32_t sample
             sample_count *= 2;
 
         while (sample_count--)
-            *buffer++ <<= shift;
+            *(uint32_t*)buffer++ <<= shift;
     }
 }
 
