@@ -11,11 +11,13 @@
 // This module provides unpacking for WavPack files prior to version 4.0,
 // not including "raw" files. As these modes are all obsolete and are no
 // longer written, this code will not be fully documented other than the
-// global functions. However, full documenation is provided in the version
+// global functions. However, full documentation is provided in the version
 // 3.97 source code. Note that this module does only the low-level sample
 // unpacking; the actual opening of the file (and obtaining information
 // from it) is handled in the unpack3_open.c module.
 
+#ifdef ENABLE_LEGACY
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -24,7 +26,7 @@
 
 #define ATTEMPT_ERROR_MUTING
 
-static int bs_open_read3 (Bitstream3 *bs, WavpackStreamReader *reader, void *id);
+static int bs_open_read3 (Bitstream3 *bs, WavpackStreamReader64 *reader, void *id);
 static uint32_t bs_unused_bytes (Bitstream3 *bs);
 static unsigned char *bs_unused_data (Bitstream3 *bs);
 static void init_words3 (WavpackStream3 *wps);
@@ -264,7 +266,7 @@ int32_t unpack_samples3 (WavpackContext *wpc, int32_t *buffer, uint32_t sample_c
     WavpackStream3 *wps = (WavpackStream3 *) wpc->stream3;
     int shift = wps->wphdr.shift, flags = wps->wphdr.flags, min_weight = 0, m = wps->dc.m, tcount;
 #ifndef NO_SEEKING
-    int points_index = wps->sample_index / ((wpc->total_samples >> 8) + 1);
+    int points_index = wps->sample_index / (((uint32_t) wpc->total_samples >> 8) + 1);
 #endif
     int32_t min_value, max_value, min_shifted, max_shifted;
     int32_t correction [2], crc = wps->dc.crc;
@@ -287,7 +289,7 @@ int32_t unpack_samples3 (WavpackContext *wpc, int32_t *buffer, uint32_t sample_c
 #endif
 
     if (wps->sample_index + sample_count > wpc->total_samples)
-        sample_count = wpc->total_samples - wps->sample_index;
+        sample_count = (uint32_t) (wpc->total_samples - wps->sample_index);
 
     if (!sample_count)
         return 0;
@@ -1172,11 +1174,11 @@ int32_t unpack_samples3 (WavpackContext *wpc, int32_t *buffer, uint32_t sample_c
             wpc->crc_errors++;
 
         if (wpc->open_flags & OPEN_WRAPPER) {
-            unsigned char *temp = malloc (1024);
+            unsigned char *temp = (unsigned char *)malloc (1024);
             uint32_t bcount;
 
             if (bs_unused_bytes (&wps->wvbits)) {
-                wpc->wrapper_data = realloc (wpc->wrapper_data, wpc->wrapper_bytes + bs_unused_bytes (&wps->wvbits));
+                wpc->wrapper_data = (unsigned char *)realloc (wpc->wrapper_data, wpc->wrapper_bytes + bs_unused_bytes (&wps->wvbits));
                 memcpy (wpc->wrapper_data + wpc->wrapper_bytes, bs_unused_data (&wps->wvbits), bs_unused_bytes (&wps->wvbits));
                 wpc->wrapper_bytes += bs_unused_bytes (&wps->wvbits);
             }
@@ -1187,7 +1189,7 @@ int32_t unpack_samples3 (WavpackContext *wpc, int32_t *buffer, uint32_t sample_c
                 if (!bcount)
                     break;
 
-                wpc->wrapper_data = realloc (wpc->wrapper_data, wpc->wrapper_bytes + bcount);
+                wpc->wrapper_data = (unsigned char *)realloc (wpc->wrapper_data, wpc->wrapper_bytes + bcount);
                 memcpy (wpc->wrapper_data + wpc->wrapper_bytes, temp, bcount);
                 wpc->wrapper_bytes += bcount;
             }
@@ -1482,7 +1484,7 @@ static int32_t FASTCALL get_word3 (WavpackStream3 *wps, int chan)
     }
 }
 
-static int FASTCALL _log2 (uint32_t avalue);
+static int FASTCALL wp3_log2 (uint32_t avalue);
 
 static int32_t FASTCALL get_word4 (WavpackStream3 *wps, int chan, int32_t *correction)
 {
@@ -1525,22 +1527,22 @@ static int32_t FASTCALL get_word4 (WavpackStream3 *wps, int chan, int32_t *corre
         int slow_log_0, slow_log_1, balance;
 
         if (wps->wphdr.flags & MONO_FLAG) {
-            wps->w4.bits_acc [0] += wps->w4.bitrate + _log2 (wps->w4.fast_level [0]) - _log2 (wps->w4.slow_level [0]) + (3 << 8);
+            wps->w4.bits_acc [0] += wps->w4.bitrate + wp3_log2 (wps->w4.fast_level [0]) - wp3_log2 (wps->w4.slow_level [0]) + (3 << 8);
 
             if (wps->w4.bits_acc [0] < 0)
                 wps->w4.bits_acc [0] = 0;
         }
         else {
-            slow_log_0 = _log2 (wps->w4.slow_level [0]);
-            slow_log_1 = _log2 (wps->w4.slow_level [1]);
+            slow_log_0 = wp3_log2 (wps->w4.slow_level [0]);
+            slow_log_1 = wp3_log2 (wps->w4.slow_level [1]);
 
             if (wps->wphdr.flags & JOINT_STEREO)
                 balance = (slow_log_1 - slow_log_0 + 257) >> 1;
             else
                 balance = (slow_log_1 - slow_log_0 + 1) >> 1;
 
-            wps->w4.bits_acc [0] += wps->w4.bitrate - balance + _log2 (wps->w4.fast_level [0]) - slow_log_0 + (3 << 8);
-            wps->w4.bits_acc [1] += wps->w4.bitrate + balance + _log2 (wps->w4.fast_level [1]) - slow_log_1 + (3 << 8);
+            wps->w4.bits_acc [0] += wps->w4.bitrate - balance + wp3_log2 (wps->w4.fast_level [0]) - slow_log_0 + (3 << 8);
+            wps->w4.bits_acc [1] += wps->w4.bitrate + balance + wp3_log2 (wps->w4.fast_level [1]) - slow_log_1 + (3 << 8);
 
             if (wps->w4.bits_acc [0] + wps->w4.bits_acc [1] < 0)
                 wps->w4.bits_acc [0] = wps->w4.bits_acc [1] = 0;
@@ -1620,7 +1622,7 @@ static int32_t FASTCALL get_word4 (WavpackStream3 *wps, int chan, int32_t *corre
 // fraction) from the supplied value. Using logarithms makes comparing
 // signal level values and calculating fractional bitrates much easier.
 
-static int FASTCALL _log2 (uint32_t avalue)
+static int FASTCALL wp3_log2 (uint32_t avalue)
 {
     int dbits;
 
@@ -1662,7 +1664,7 @@ static void bs_read3 (Bitstream3 *bs)
 // data is located.  A return value of TRUE indicates an error in
 // allocating buffer space.
 
-static int bs_open_read3 (Bitstream3 *bs, WavpackStreamReader *reader, void *id)
+static int bs_open_read3 (Bitstream3 *bs, WavpackStreamReader64 *reader, void *id)
 {
     bs->fpos = (bs->reader = reader)->get_pos (bs->id = id);
 
@@ -1697,3 +1699,5 @@ static unsigned char *bs_unused_data (Bitstream3 *bs)
     return bs->ptr;
 }
 
+#endif  // ENABLE_LEGACY
+
